from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.currency import Currency
from ..types import UNSET, Unset

C = TypeVar("C", bound="InvoiceLineItem")


@attr.s(auto_attribs=True)
class InvoiceLineItem:
    """An invoice line item."""  # noqa: E501

    amount: Union[Unset, float] = UNSET
    currency: Union[Unset, Currency] = UNSET
    description: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    invoice_item: Union[Unset, str] = UNSET
    metadata: Union[Unset, Any] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        amount = self.amount
        if not isinstance(self.currency, Unset):
            currency = self.currency
        description = self.description
        id = self.id
        invoice_item = self.invoice_item
        metadata = self.metadata

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if amount is not UNSET:
            field_dict["amount"] = amount
        if currency is not UNSET:
            field_dict["currency"] = currency
        if description is not UNSET:
            field_dict["description"] = description
        if id is not UNSET:
            field_dict["id"] = id
        if invoice_item is not UNSET:
            field_dict["invoice_item"] = invoice_item
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict

    @classmethod
    def from_dict(cls: Type[C], src_dict: Dict[str, Any]) -> C:
        d = src_dict.copy()
        amount = d.pop("amount", UNSET)

        _currency = d.pop("currency", UNSET)
        currency: Union[Unset, Currency]
        if isinstance(_currency, Unset):
            currency = UNSET
        else:
            currency = _currency  # type: ignore[arg-type]

        description = d.pop("description", UNSET)

        id = d.pop("id", UNSET)

        invoice_item = d.pop("invoice_item", UNSET)

        metadata = d.pop("metadata", UNSET)

        invoice_line_item = cls(
            amount=amount,
            currency=currency,
            description=description,
            id=id,
            invoice_item=invoice_item,
            metadata=metadata,
        )

        invoice_line_item.additional_properties = d
        return invoice_line_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr
from dateutil.parser import isoparse

from ..models.currency import Currency
from ..models.invoice_status import InvoiceStatus
from ..types import UNSET, Unset

P = TypeVar("P", bound="Invoice")


@attr.s(auto_attribs=True)
class Invoice:
    """An invoice."""  # noqa: E501

    amount_due: Union[Unset, float] = UNSET
    amount_paid: Union[Unset, float] = UNSET
    amount_remaining: Union[Unset, float] = UNSET
    attempt_count: Union[Unset, int] = UNSET
    attempted: Union[Unset, bool] = False
    created_at: Union[Unset, datetime.datetime] = UNSET
    currency: Union[Unset, Currency] = UNSET
    customer_email: Union[Unset, str] = UNSET
    customer_id: Union[Unset, str] = UNSET
    default_payment_method: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    from ..models.discount import Discount

    discounts: Union[Unset, List[Discount]] = UNSET
    id: Union[Unset, str] = UNSET
    from ..models.invoice_line_item import InvoiceLineItem

    lines: Union[Unset, List[InvoiceLineItem]] = UNSET
    metadata: Union[Unset, Any] = UNSET
    number: Union[Unset, str] = UNSET
    paid: Union[Unset, bool] = False
    pdf: Union[Unset, str] = UNSET
    receipt_number: Union[Unset, str] = UNSET
    statement_descriptor: Union[Unset, str] = UNSET
    status: Union[Unset, InvoiceStatus] = UNSET
    subtotal: Union[Unset, float] = UNSET
    tax: Union[Unset, float] = UNSET
    total: Union[Unset, float] = UNSET
    url: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        amount_due = self.amount_due
        amount_paid = self.amount_paid
        amount_remaining = self.amount_remaining
        attempt_count = self.attempt_count
        attempted = self.attempted
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()
        if not isinstance(self.currency, Unset):
            currency = self.currency
        customer_email = self.customer_email
        customer_id = self.customer_id
        default_payment_method = self.default_payment_method
        description = self.description
        from ..models.discount import Discount

        discounts: Union[Unset, List[Discount]] = UNSET
        if not isinstance(self.discounts, Unset):
            discounts = self.discounts
        id = self.id
        from ..models.invoice_line_item import InvoiceLineItem

        lines: Union[Unset, List[InvoiceLineItem]] = UNSET
        if not isinstance(self.lines, Unset):
            lines = self.lines
        metadata = self.metadata
        number = self.number
        paid = self.paid
        pdf = self.pdf
        receipt_number = self.receipt_number
        statement_descriptor = self.statement_descriptor
        if not isinstance(self.status, Unset):
            status = self.status
        subtotal = self.subtotal
        tax = self.tax
        total = self.total
        url = self.url

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if amount_due is not UNSET:
            field_dict["amount_due"] = amount_due
        if amount_paid is not UNSET:
            field_dict["amount_paid"] = amount_paid
        if amount_remaining is not UNSET:
            field_dict["amount_remaining"] = amount_remaining
        if attempt_count is not UNSET:
            field_dict["attempt_count"] = attempt_count
        if attempted is not UNSET:
            field_dict["attempted"] = attempted
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if currency is not UNSET:
            field_dict["currency"] = currency
        if customer_email is not UNSET:
            field_dict["customer_email"] = customer_email
        if customer_id is not UNSET:
            field_dict["customer_id"] = customer_id
        if default_payment_method is not UNSET:
            field_dict["default_payment_method"] = default_payment_method
        if description is not UNSET:
            field_dict["description"] = description
        if discounts is not UNSET:
            field_dict["discounts"] = discounts
        if id is not UNSET:
            field_dict["id"] = id
        if lines is not UNSET:
            field_dict["lines"] = lines
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if number is not UNSET:
            field_dict["number"] = number
        if paid is not UNSET:
            field_dict["paid"] = paid
        if pdf is not UNSET:
            field_dict["pdf"] = pdf
        if receipt_number is not UNSET:
            field_dict["receipt_number"] = receipt_number
        if statement_descriptor is not UNSET:
            field_dict["statement_descriptor"] = statement_descriptor
        if status is not UNSET:
            field_dict["status"] = status
        if subtotal is not UNSET:
            field_dict["subtotal"] = subtotal
        if tax is not UNSET:
            field_dict["tax"] = tax
        if total is not UNSET:
            field_dict["total"] = total
        if url is not UNSET:
            field_dict["url"] = url

        return field_dict

    @classmethod
    def from_dict(cls: Type[P], src_dict: Dict[str, Any]) -> P:
        d = src_dict.copy()
        amount_due = d.pop("amount_due", UNSET)

        amount_paid = d.pop("amount_paid", UNSET)

        amount_remaining = d.pop("amount_remaining", UNSET)

        attempt_count = d.pop("attempt_count", UNSET)

        attempted = d.pop("attempted", UNSET)

        _created_at = d.pop("created_at", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _currency = d.pop("currency", UNSET)
        currency: Union[Unset, Currency]
        if isinstance(_currency, Unset):
            currency = UNSET
        else:
            currency = _currency  # type: ignore[arg-type]

        customer_email = d.pop("customer_email", UNSET)

        customer_id = d.pop("customer_id", UNSET)

        default_payment_method = d.pop("default_payment_method", UNSET)

        description = d.pop("description", UNSET)

        from ..models.discount import Discount

        discounts = cast(List[Discount], d.pop("discounts", UNSET))

        id = d.pop("id", UNSET)

        from ..models.invoice_line_item import InvoiceLineItem

        lines = cast(List[InvoiceLineItem], d.pop("lines", UNSET))

        metadata = d.pop("metadata", UNSET)
        number = d.pop("number", UNSET)

        paid = d.pop("paid", UNSET)

        pdf = d.pop("pdf", UNSET)

        receipt_number = d.pop("receipt_number", UNSET)

        statement_descriptor = d.pop("statement_descriptor", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, InvoiceStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = _status  # type: ignore[arg-type]

        subtotal = d.pop("subtotal", UNSET)

        tax = d.pop("tax", UNSET)

        total = d.pop("total", UNSET)

        url = d.pop("url", UNSET)

        invoice = cls(
            amount_due=amount_due,
            amount_paid=amount_paid,
            amount_remaining=amount_remaining,
            attempt_count=attempt_count,
            attempted=attempted,
            created_at=created_at,
            currency=currency,
            customer_email=customer_email,
            customer_id=customer_id,
            default_payment_method=default_payment_method,
            description=description,
            discounts=discounts,
            id=id,
            lines=lines,
            metadata=metadata,
            number=number,
            paid=paid,
            pdf=pdf,
            receipt_number=receipt_number,
            statement_descriptor=statement_descriptor,
            status=status,
            subtotal=subtotal,
            tax=tax,
            total=total,
            url=url,
        )

        invoice.additional_properties = d
        return invoice

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ApiCallStatus(str, Enum):
    """The status of an async API call."""  # noqa: E501

    """# The async API call is queued. """  # noqa: E501
    QUEUED = "Queued"
    """# The async API call was uploaded to be converted. """  # noqa: E501
    UPLOADED = "Uploaded"
    """# The async API call is in progress. """  # noqa: E501
    IN_PROGRESS = "In Progress"
    """# The async API call has completed. """  # noqa: E501
    COMPLETED = "Completed"
    """# The async API call has failed. """  # noqa: E501
    FAILED = "Failed"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.axis_direction_pair import AxisDirectionPair
from ..types import UNSET, Unset

V = TypeVar("V", bound="System")


@attr.s(auto_attribs=True)
class System:
    """Co-ordinate system definition.

    The `up` axis must be orthogonal to the `forward` axis.

    See [cglearn.eu] for background reading.

    [cglearn.eu](https://cglearn.eu/pub/computer-graphics/introduction-to-geometry#material-coordinate-systems-1)"""  # noqa: E501

    forward: Union[Unset, AxisDirectionPair] = UNSET
    up: Union[Unset, AxisDirectionPair] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        if not isinstance(self.forward, Unset):
            forward = self.forward
        if not isinstance(self.up, Unset):
            up = self.up

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if forward is not UNSET:
            field_dict["forward"] = forward
        if up is not UNSET:
            field_dict["up"] = up

        return field_dict

    @classmethod
    def from_dict(cls: Type[V], src_dict: Dict[str, Any]) -> V:
        d = src_dict.copy()
        _forward = d.pop("forward", UNSET)
        forward: Union[Unset, AxisDirectionPair]
        if isinstance(_forward, Unset):
            forward = UNSET
        else:
            forward = AxisDirectionPair(_forward)

        _up = d.pop("up", UNSET)
        up: Union[Unset, AxisDirectionPair]
        if isinstance(_up, Unset):
            up = UNSET
        else:
            up = AxisDirectionPair(_up)

        system = cls(
            forward=forward,
            up=up,
        )

        system.additional_properties = d
        return system

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.coupon import Coupon
from ..types import UNSET, Unset

V = TypeVar("V", bound="Discount")


@attr.s(auto_attribs=True)
class Discount:
    """The resource representing a Discount."""  # noqa: E501

    coupon: Union[Unset, Coupon] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        if not isinstance(self.coupon, Unset):
            coupon = self.coupon

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if coupon is not UNSET:
            field_dict["coupon"] = coupon

        return field_dict

    @classmethod
    def from_dict(cls: Type[V], src_dict: Dict[str, Any]) -> V:
        d = src_dict.copy()
        _coupon = d.pop("coupon", UNSET)
        coupon: Union[Unset, Coupon]
        if isinstance(_coupon, Unset):
            coupon = UNSET
        else:
            coupon = Coupon(_coupon)

        discount = cls(
            coupon=coupon,
        )

        discount.additional_properties = d
        return discount

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.commit import Commit
from ..models.plugins_info import PluginsInfo
from ..models.registry_service_config import RegistryServiceConfig
from ..models.system_info_cgroup_driver_enum import SystemInfoCgroupDriverEnum
from ..models.system_info_cgroup_version_enum import SystemInfoCgroupVersionEnum
from ..models.system_info_isolation_enum import SystemInfoIsolationEnum
from ..types import UNSET, Unset

L = TypeVar("L", bound="DockerSystemInfo")


@attr.s(auto_attribs=True)
class DockerSystemInfo:
    """Docker system info."""  # noqa: E501

    architecture: Union[Unset, str] = UNSET
    bridge_nf_ip6tables: Union[Unset, bool] = False
    bridge_nf_iptables: Union[Unset, bool] = False
    cgroup_driver: Union[Unset, SystemInfoCgroupDriverEnum] = UNSET
    cgroup_version: Union[Unset, SystemInfoCgroupVersionEnum] = UNSET
    cluster_advertise: Union[Unset, str] = UNSET
    cluster_store: Union[Unset, str] = UNSET
    containerd_commit: Union[Unset, Commit] = UNSET
    containers: Union[Unset, int] = UNSET
    containers_paused: Union[Unset, int] = UNSET
    containers_running: Union[Unset, int] = UNSET
    containers_stopped: Union[Unset, int] = UNSET
    cpu_cfs_period: Union[Unset, bool] = False
    cpu_cfs_quota: Union[Unset, bool] = False
    cpu_set: Union[Unset, bool] = False
    cpu_shares: Union[Unset, bool] = False
    debug: Union[Unset, bool] = False
    from ..models.system_info_default_address_pools import SystemInfoDefaultAddressPools

    default_address_pools: Union[Unset, List[SystemInfoDefaultAddressPools]] = UNSET
    default_runtime: Union[Unset, str] = UNSET
    docker_root_dir: Union[Unset, str] = UNSET
    driver: Union[Unset, str] = UNSET
    driver_status: Union[Unset, List[List[str]]] = UNSET
    experimental_build: Union[Unset, bool] = False
    http_proxy: Union[Unset, str] = UNSET
    https_proxy: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    images: Union[Unset, int] = UNSET
    index_server_address: Union[Unset, str] = UNSET
    init_binary: Union[Unset, str] = UNSET
    init_commit: Union[Unset, Commit] = UNSET
    ipv4_forwarding: Union[Unset, bool] = False
    isolation: Union[Unset, SystemInfoIsolationEnum] = UNSET
    kernel_memory: Union[Unset, bool] = False
    kernel_memory_tcp: Union[Unset, bool] = False
    kernel_version: Union[Unset, str] = UNSET
    labels: Union[Unset, List[str]] = UNSET
    live_restore_enabled: Union[Unset, bool] = False
    logging_driver: Union[Unset, str] = UNSET
    mem_total: Union[Unset, int] = UNSET
    memory_limit: Union[Unset, bool] = False
    n_events_listener: Union[Unset, int] = UNSET
    n_fd: Union[Unset, int] = UNSET
    name: Union[Unset, str] = UNSET
    ncpu: Union[Unset, int] = UNSET
    no_proxy: Union[Unset, str] = UNSET
    oom_kill_disable: Union[Unset, bool] = False
    operating_system: Union[Unset, str] = UNSET
    os_type: Union[Unset, str] = UNSET
    os_version: Union[Unset, str] = UNSET
    pids_limit: Union[Unset, bool] = False
    plugins: Union[Unset, PluginsInfo] = UNSET
    product_license: Union[Unset, str] = UNSET
    registry_config: Union[Unset, RegistryServiceConfig] = UNSET
    runc_commit: Union[Unset, Commit] = UNSET
    runtimes: Union[Unset, Any] = UNSET
    security_options: Union[Unset, List[str]] = UNSET
    server_version: Union[Unset, str] = UNSET
    swap_limit: Union[Unset, bool] = False
    system_time: Union[Unset, str] = UNSET
    warnings: Union[Unset, List[str]] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        architecture = self.architecture
        bridge_nf_ip6tables = self.bridge_nf_ip6tables
        bridge_nf_iptables = self.bridge_nf_iptables
        if not isinstance(self.cgroup_driver, Unset):
            cgroup_driver = self.cgroup_driver
        if not isinstance(self.cgroup_version, Unset):
            cgroup_version = self.cgroup_version
        cluster_advertise = self.cluster_advertise
        cluster_store = self.cluster_store
        if not isinstance(self.containerd_commit, Unset):
            containerd_commit = self.containerd_commit
        containers = self.containers
        containers_paused = self.containers_paused
        containers_running = self.containers_running
        containers_stopped = self.containers_stopped
        cpu_cfs_period = self.cpu_cfs_period
        cpu_cfs_quota = self.cpu_cfs_quota
        cpu_set = self.cpu_set
        cpu_shares = self.cpu_shares
        debug = self.debug
        from ..models.system_info_default_address_pools import (
            SystemInfoDefaultAddressPools,
        )

        default_address_pools: Union[Unset, List[SystemInfoDefaultAddressPools]] = UNSET
        if not isinstance(self.default_address_pools, Unset):
            default_address_pools = self.default_address_pools
        default_runtime = self.default_runtime
        docker_root_dir = self.docker_root_dir
        driver = self.driver
        driver_status: Union[Unset, List[List[str]]] = UNSET
        if not isinstance(self.driver_status, Unset):
            driver_status = self.driver_status
        experimental_build = self.experimental_build
        http_proxy = self.http_proxy
        https_proxy = self.https_proxy
        id = self.id
        images = self.images
        index_server_address = self.index_server_address
        init_binary = self.init_binary
        if not isinstance(self.init_commit, Unset):
            init_commit = self.init_commit
        ipv4_forwarding = self.ipv4_forwarding
        if not isinstance(self.isolation, Unset):
            isolation = self.isolation
        kernel_memory = self.kernel_memory
        kernel_memory_tcp = self.kernel_memory_tcp
        kernel_version = self.kernel_version
        labels: Union[Unset, List[str]] = UNSET
        if not isinstance(self.labels, Unset):
            labels = self.labels
        live_restore_enabled = self.live_restore_enabled
        logging_driver = self.logging_driver
        mem_total = self.mem_total
        memory_limit = self.memory_limit
        n_events_listener = self.n_events_listener
        n_fd = self.n_fd
        name = self.name
        ncpu = self.ncpu
        no_proxy = self.no_proxy
        oom_kill_disable = self.oom_kill_disable
        operating_system = self.operating_system
        os_type = self.os_type
        os_version = self.os_version
        pids_limit = self.pids_limit
        if not isinstance(self.plugins, Unset):
            plugins = self.plugins
        product_license = self.product_license
        if not isinstance(self.registry_config, Unset):
            registry_config = self.registry_config
        if not isinstance(self.runc_commit, Unset):
            runc_commit = self.runc_commit
        runtimes = self.runtimes
        security_options: Union[Unset, List[str]] = UNSET
        if not isinstance(self.security_options, Unset):
            security_options = self.security_options
        server_version = self.server_version
        swap_limit = self.swap_limit
        system_time = self.system_time
        warnings: Union[Unset, List[str]] = UNSET
        if not isinstance(self.warnings, Unset):
            warnings = self.warnings

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if architecture is not UNSET:
            field_dict["architecture"] = architecture
        if bridge_nf_ip6tables is not UNSET:
            field_dict["bridge_nf_ip6tables"] = bridge_nf_ip6tables
        if bridge_nf_iptables is not UNSET:
            field_dict["bridge_nf_iptables"] = bridge_nf_iptables
        if cgroup_driver is not UNSET:
            field_dict["cgroup_driver"] = cgroup_driver
        if cgroup_version is not UNSET:
            field_dict["cgroup_version"] = cgroup_version
        if cluster_advertise is not UNSET:
            field_dict["cluster_advertise"] = cluster_advertise
        if cluster_store is not UNSET:
            field_dict["cluster_store"] = cluster_store
        if containerd_commit is not UNSET:
            field_dict["containerd_commit"] = containerd_commit
        if containers is not UNSET:
            field_dict["containers"] = containers
        if containers_paused is not UNSET:
            field_dict["containers_paused"] = containers_paused
        if containers_running is not UNSET:
            field_dict["containers_running"] = containers_running
        if containers_stopped is not UNSET:
            field_dict["containers_stopped"] = containers_stopped
        if cpu_cfs_period is not UNSET:
            field_dict["cpu_cfs_period"] = cpu_cfs_period
        if cpu_cfs_quota is not UNSET:
            field_dict["cpu_cfs_quota"] = cpu_cfs_quota
        if cpu_set is not UNSET:
            field_dict["cpu_set"] = cpu_set
        if cpu_shares is not UNSET:
            field_dict["cpu_shares"] = cpu_shares
        if debug is not UNSET:
            field_dict["debug"] = debug
        if default_address_pools is not UNSET:
            field_dict["default_address_pools"] = default_address_pools
        if default_runtime is not UNSET:
            field_dict["default_runtime"] = default_runtime
        if docker_root_dir is not UNSET:
            field_dict["docker_root_dir"] = docker_root_dir
        if driver is not UNSET:
            field_dict["driver"] = driver
        if driver_status is not UNSET:
            field_dict["driver_status"] = driver_status
        if experimental_build is not UNSET:
            field_dict["experimental_build"] = experimental_build
        if http_proxy is not UNSET:
            field_dict["http_proxy"] = http_proxy
        if https_proxy is not UNSET:
            field_dict["https_proxy"] = https_proxy
        if id is not UNSET:
            field_dict["id"] = id
        if images is not UNSET:
            field_dict["images"] = images
        if index_server_address is not UNSET:
            field_dict["index_server_address"] = index_server_address
        if init_binary is not UNSET:
            field_dict["init_binary"] = init_binary
        if init_commit is not UNSET:
            field_dict["init_commit"] = init_commit
        if ipv4_forwarding is not UNSET:
            field_dict["ipv4_forwarding"] = ipv4_forwarding
        if isolation is not UNSET:
            field_dict["isolation"] = isolation
        if kernel_memory is not UNSET:
            field_dict["kernel_memory"] = kernel_memory
        if kernel_memory_tcp is not UNSET:
            field_dict["kernel_memory_tcp"] = kernel_memory_tcp
        if kernel_version is not UNSET:
            field_dict["kernel_version"] = kernel_version
        if labels is not UNSET:
            field_dict["labels"] = labels
        if live_restore_enabled is not UNSET:
            field_dict["live_restore_enabled"] = live_restore_enabled
        if logging_driver is not UNSET:
            field_dict["logging_driver"] = logging_driver
        if mem_total is not UNSET:
            field_dict["mem_total"] = mem_total
        if memory_limit is not UNSET:
            field_dict["memory_limit"] = memory_limit
        if n_events_listener is not UNSET:
            field_dict["n_events_listener"] = n_events_listener
        if n_fd is not UNSET:
            field_dict["n_fd"] = n_fd
        if name is not UNSET:
            field_dict["name"] = name
        if ncpu is not UNSET:
            field_dict["ncpu"] = ncpu
        if no_proxy is not UNSET:
            field_dict["no_proxy"] = no_proxy
        if oom_kill_disable is not UNSET:
            field_dict["oom_kill_disable"] = oom_kill_disable
        if operating_system is not UNSET:
            field_dict["operating_system"] = operating_system
        if os_type is not UNSET:
            field_dict["os_type"] = os_type
        if os_version is not UNSET:
            field_dict["os_version"] = os_version
        if pids_limit is not UNSET:
            field_dict["pids_limit"] = pids_limit
        if plugins is not UNSET:
            field_dict["plugins"] = plugins
        if product_license is not UNSET:
            field_dict["product_license"] = product_license
        if registry_config is not UNSET:
            field_dict["registry_config"] = registry_config
        if runc_commit is not UNSET:
            field_dict["runc_commit"] = runc_commit
        if runtimes is not UNSET:
            field_dict["runtimes"] = runtimes
        if security_options is not UNSET:
            field_dict["security_options"] = security_options
        if server_version is not UNSET:
            field_dict["server_version"] = server_version
        if swap_limit is not UNSET:
            field_dict["swap_limit"] = swap_limit
        if system_time is not UNSET:
            field_dict["system_time"] = system_time
        if warnings is not UNSET:
            field_dict["warnings"] = warnings

        return field_dict

    @classmethod
    def from_dict(cls: Type[L], src_dict: Dict[str, Any]) -> L:
        d = src_dict.copy()
        architecture = d.pop("architecture", UNSET)

        bridge_nf_ip6tables = d.pop("bridge_nf_ip6tables", UNSET)

        bridge_nf_iptables = d.pop("bridge_nf_iptables", UNSET)

        _cgroup_driver = d.pop("cgroup_driver", UNSET)
        cgroup_driver: Union[Unset, SystemInfoCgroupDriverEnum]
        if isinstance(_cgroup_driver, Unset):
            cgroup_driver = UNSET
        else:
            cgroup_driver = SystemInfoCgroupDriverEnum(_cgroup_driver)

        _cgroup_version = d.pop("cgroup_version", UNSET)
        cgroup_version: Union[Unset, SystemInfoCgroupVersionEnum]
        if isinstance(_cgroup_version, Unset):
            cgroup_version = UNSET
        else:
            cgroup_version = SystemInfoCgroupVersionEnum(_cgroup_version)

        cluster_advertise = d.pop("cluster_advertise", UNSET)

        cluster_store = d.pop("cluster_store", UNSET)

        _containerd_commit = d.pop("containerd_commit", UNSET)
        containerd_commit: Union[Unset, Commit]
        if isinstance(_containerd_commit, Unset):
            containerd_commit = UNSET
        else:
            containerd_commit = Commit(_containerd_commit)

        containers = d.pop("containers", UNSET)

        containers_paused = d.pop("containers_paused", UNSET)

        containers_running = d.pop("containers_running", UNSET)

        containers_stopped = d.pop("containers_stopped", UNSET)

        cpu_cfs_period = d.pop("cpu_cfs_period", UNSET)

        cpu_cfs_quota = d.pop("cpu_cfs_quota", UNSET)

        cpu_set = d.pop("cpu_set", UNSET)

        cpu_shares = d.pop("cpu_shares", UNSET)

        debug = d.pop("debug", UNSET)

        from ..models.system_info_default_address_pools import (
            SystemInfoDefaultAddressPools,
        )

        default_address_pools = cast(
            List[SystemInfoDefaultAddressPools], d.pop("default_address_pools", UNSET)
        )

        default_runtime = d.pop("default_runtime", UNSET)

        docker_root_dir = d.pop("docker_root_dir", UNSET)

        driver = d.pop("driver", UNSET)

        driver_status = cast(List[List[str]], d.pop("driver_status", UNSET))

        experimental_build = d.pop("experimental_build", UNSET)

        http_proxy = d.pop("http_proxy", UNSET)

        https_proxy = d.pop("https_proxy", UNSET)

        id = d.pop("id", UNSET)

        images = d.pop("images", UNSET)

        index_server_address = d.pop("index_server_address", UNSET)

        init_binary = d.pop("init_binary", UNSET)

        _init_commit = d.pop("init_commit", UNSET)
        init_commit: Union[Unset, Commit]
        if isinstance(_init_commit, Unset):
            init_commit = UNSET
        else:
            init_commit = Commit(_init_commit)

        ipv4_forwarding = d.pop("ipv4_forwarding", UNSET)

        _isolation = d.pop("isolation", UNSET)
        isolation: Union[Unset, SystemInfoIsolationEnum]
        if isinstance(_isolation, Unset):
            isolation = UNSET
        else:
            isolation = SystemInfoIsolationEnum(_isolation)

        kernel_memory = d.pop("kernel_memory", UNSET)

        kernel_memory_tcp = d.pop("kernel_memory_tcp", UNSET)

        kernel_version = d.pop("kernel_version", UNSET)

        labels = cast(List[str], d.pop("labels", UNSET))

        live_restore_enabled = d.pop("live_restore_enabled", UNSET)

        logging_driver = d.pop("logging_driver", UNSET)

        mem_total = d.pop("mem_total", UNSET)

        memory_limit = d.pop("memory_limit", UNSET)

        n_events_listener = d.pop("n_events_listener", UNSET)

        n_fd = d.pop("n_fd", UNSET)

        name = d.pop("name", UNSET)

        ncpu = d.pop("ncpu", UNSET)

        no_proxy = d.pop("no_proxy", UNSET)

        oom_kill_disable = d.pop("oom_kill_disable", UNSET)

        operating_system = d.pop("operating_system", UNSET)

        os_type = d.pop("os_type", UNSET)

        os_version = d.pop("os_version", UNSET)

        pids_limit = d.pop("pids_limit", UNSET)

        _plugins = d.pop("plugins", UNSET)
        plugins: Union[Unset, PluginsInfo]
        if isinstance(_plugins, Unset):
            plugins = UNSET
        else:
            plugins = PluginsInfo(_plugins)

        product_license = d.pop("product_license", UNSET)

        _registry_config = d.pop("registry_config", UNSET)
        registry_config: Union[Unset, RegistryServiceConfig]
        if isinstance(_registry_config, Unset):
            registry_config = UNSET
        else:
            registry_config = RegistryServiceConfig(_registry_config)

        _runc_commit = d.pop("runc_commit", UNSET)
        runc_commit: Union[Unset, Commit]
        if isinstance(_runc_commit, Unset):
            runc_commit = UNSET
        else:
            runc_commit = Commit(_runc_commit)

        runtimes = d.pop("runtimes", UNSET)
        security_options = cast(List[str], d.pop("security_options", UNSET))

        server_version = d.pop("server_version", UNSET)

        swap_limit = d.pop("swap_limit", UNSET)

        system_time = d.pop("system_time", UNSET)

        warnings = cast(List[str], d.pop("warnings", UNSET))

        docker_system_info = cls(
            architecture=architecture,
            bridge_nf_ip6tables=bridge_nf_ip6tables,
            bridge_nf_iptables=bridge_nf_iptables,
            cgroup_driver=cgroup_driver,
            cgroup_version=cgroup_version,
            cluster_advertise=cluster_advertise,
            cluster_store=cluster_store,
            containerd_commit=containerd_commit,
            containers=containers,
            containers_paused=containers_paused,
            containers_running=containers_running,
            containers_stopped=containers_stopped,
            cpu_cfs_period=cpu_cfs_period,
            cpu_cfs_quota=cpu_cfs_quota,
            cpu_set=cpu_set,
            cpu_shares=cpu_shares,
            debug=debug,
            default_address_pools=default_address_pools,
            default_runtime=default_runtime,
            docker_root_dir=docker_root_dir,
            driver=driver,
            driver_status=driver_status,
            experimental_build=experimental_build,
            http_proxy=http_proxy,
            https_proxy=https_proxy,
            id=id,
            images=images,
            index_server_address=index_server_address,
            init_binary=init_binary,
            init_commit=init_commit,
            ipv4_forwarding=ipv4_forwarding,
            isolation=isolation,
            kernel_memory=kernel_memory,
            kernel_memory_tcp=kernel_memory_tcp,
            kernel_version=kernel_version,
            labels=labels,
            live_restore_enabled=live_restore_enabled,
            logging_driver=logging_driver,
            mem_total=mem_total,
            memory_limit=memory_limit,
            n_events_listener=n_events_listener,
            n_fd=n_fd,
            name=name,
            ncpu=ncpu,
            no_proxy=no_proxy,
            oom_kill_disable=oom_kill_disable,
            operating_system=operating_system,
            os_type=os_type,
            os_version=os_version,
            pids_limit=pids_limit,
            plugins=plugins,
            product_license=product_license,
            registry_config=registry_config,
            runc_commit=runc_commit,
            runtimes=runtimes,
            security_options=security_options,
            server_version=server_version,
            swap_limit=swap_limit,
            system_time=system_time,
            warnings=warnings,
        )

        docker_system_info.additional_properties = d
        return docker_system_info

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.storage import Storage
from ..models.system import System
from ..types import UNSET, Unset

D = TypeVar("D", bound="Gltf")


@attr.s(auto_attribs=True)
class Gltf:
    """glTF 2.0. We refer to this as glTF since that is how our customers refer to it, although by default it will be in binary format and thus technically (glb). If you prefer ascii output, you can set that option for the export."""  # noqa: E501

    storage: Union[Unset, Storage] = UNSET
    type: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        if not isinstance(self.storage, Unset):
            storage = self.storage
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if storage is not UNSET:
            field_dict["storage"] = storage
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[D], src_dict: Dict[str, Any]) -> D:
        d = src_dict.copy()
        _storage = d.pop("storage", UNSET)
        storage: Union[Unset, Storage]
        if isinstance(_storage, Unset):
            storage = UNSET
        else:
            storage = _storage  # type: ignore[arg-type]

        type = d.pop("type", UNSET)

        gltf = cls(
            storage=storage,
            type=type,
        )

        gltf.additional_properties = d
        return gltf

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties


Y = TypeVar("Y", bound="Obj")


@attr.s(auto_attribs=True)
class Obj:
    """Wavefront OBJ format."""  # noqa: E501

    coords: Union[Unset, System] = UNSET
    type: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        if not isinstance(self.coords, Unset):
            coords = self.coords
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if coords is not UNSET:
            field_dict["coords"] = coords
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[Y], src_dict: Dict[str, Any]) -> Y:
        d = src_dict.copy()
        _coords = d.pop("coords", UNSET)
        coords: Union[Unset, System]
        if isinstance(_coords, Unset):
            coords = UNSET
        else:
            coords = System(_coords)

        type = d.pop("type", UNSET)

        obj = cls(
            coords=coords,
            type=type,
        )

        obj.additional_properties = d
        return obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties


Y = TypeVar("Y", bound="Step")


@attr.s(auto_attribs=True)
class Step:
    """ISO 10303-21 (STEP) format."""  # noqa: E501

    coords: Union[Unset, System] = UNSET
    type: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        if not isinstance(self.coords, Unset):
            coords = self.coords
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if coords is not UNSET:
            field_dict["coords"] = coords
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[Y], src_dict: Dict[str, Any]) -> Y:
        d = src_dict.copy()
        _coords = d.pop("coords", UNSET)
        coords: Union[Unset, System]
        if isinstance(_coords, Unset):
            coords = UNSET
        else:
            coords = System(_coords)

        type = d.pop("type", UNSET)

        step = cls(
            coords=coords,
            type=type,
        )

        step.additional_properties = d
        return step

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties


D = TypeVar("D", bound="Stl")


@attr.s(auto_attribs=True)
class Stl:
    """*ST**ereo**L**ithography format."""  # noqa: E501

    coords: Union[Unset, System] = UNSET
    storage: Union[Unset, Storage] = UNSET
    type: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        if not isinstance(self.coords, Unset):
            coords = self.coords
        if not isinstance(self.storage, Unset):
            storage = self.storage
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if coords is not UNSET:
            field_dict["coords"] = coords
        if storage is not UNSET:
            field_dict["storage"] = storage
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[D], src_dict: Dict[str, Any]) -> D:
        d = src_dict.copy()
        _coords = d.pop("coords", UNSET)
        coords: Union[Unset, System]
        if isinstance(_coords, Unset):
            coords = UNSET
        else:
            coords = System(_coords)

        _storage = d.pop("storage", UNSET)
        storage: Union[Unset, Storage]
        if isinstance(_storage, Unset):
            storage = UNSET
        else:
            storage = _storage  # type: ignore[arg-type]

        type = d.pop("type", UNSET)

        stl = cls(
            coords=coords,
            storage=storage,
            type=type,
        )

        stl.additional_properties = d
        return stl

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties


OutputFormat = Union[Gltf, Obj, Step, Stl]



from enum import Enum


class CodeLanguage(str, Enum):
    """The language code is written in."""  # noqa: E501

    """# The `go` programming language. """  # noqa: E501
    GO = "go"
    """# The `python` programming language. """  # noqa: E501
    PYTHON = "python"
    """# The `node` programming language. """  # noqa: E501
    NODE = "node"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.axis import Axis
from ..models.direction import Direction
from ..types import UNSET, Unset

X = TypeVar("X", bound="AxisDirectionPair")


@attr.s(auto_attribs=True)
class AxisDirectionPair:
    """An [`Axis`] paired with a [`Direction`]."""  # noqa: E501

    axis: Union[Unset, Axis] = UNSET
    direction: Union[Unset, Direction] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        if not isinstance(self.axis, Unset):
            axis = self.axis
        if not isinstance(self.direction, Unset):
            direction = self.direction

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if axis is not UNSET:
            field_dict["axis"] = axis
        if direction is not UNSET:
            field_dict["direction"] = direction

        return field_dict

    @classmethod
    def from_dict(cls: Type[X], src_dict: Dict[str, Any]) -> X:
        d = src_dict.copy()
        _axis = d.pop("axis", UNSET)
        axis: Union[Unset, Axis]
        if isinstance(_axis, Unset):
            axis = UNSET
        else:
            axis = _axis  # type: ignore[arg-type]

        _direction = d.pop("direction", UNSET)
        direction: Union[Unset, Direction]
        if isinstance(_direction, Unset):
            direction = UNSET
        else:
            direction = _direction  # type: ignore[arg-type]

        axis_direction_pair = cls(
            axis=axis,
            direction=direction,
        )

        axis_direction_pair.additional_properties = d
        return axis_direction_pair

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.new_address import NewAddress
from ..types import UNSET, Unset

L = TypeVar("L", bound="BillingInfo")


@attr.s(auto_attribs=True)
class BillingInfo:
    """The billing information for payments."""  # noqa: E501

    address: Union[Unset, NewAddress] = UNSET
    name: Union[Unset, str] = UNSET
    phone: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        if not isinstance(self.address, Unset):
            address = self.address
        name = self.name
        phone = self.phone

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if address is not UNSET:
            field_dict["address"] = address
        if name is not UNSET:
            field_dict["name"] = name
        if phone is not UNSET:
            field_dict["phone"] = phone

        return field_dict

    @classmethod
    def from_dict(cls: Type[L], src_dict: Dict[str, Any]) -> L:
        d = src_dict.copy()
        _address = d.pop("address", UNSET)
        address: Union[Unset, NewAddress]
        if isinstance(_address, Unset):
            address = UNSET
        else:
            address = NewAddress(_address)

        name = d.pop("name", UNSET)

        phone = d.pop("phone", UNSET)

        billing_info = cls(
            address=address,
            name=name,
            phone=phone,
        )

        billing_info.additional_properties = d
        return billing_info

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.cluster import Cluster
from ..models.gateway import Gateway
from ..models.jetstream import Jetstream
from ..models.leaf_node import LeafNode
from ..types import UNSET, Unset

M = TypeVar("M", bound="Connection")


@attr.s(auto_attribs=True)
class Connection:
    """Metadata about a pub-sub connection.

    This is mostly used for internal purposes and debugging."""  # noqa: E501

    auth_timeout: Union[Unset, int] = UNSET
    cluster: Union[Unset, Cluster] = UNSET
    config_load_time: Union[Unset, datetime.datetime] = UNSET
    connections: Union[Unset, int] = UNSET
    cores: Union[Unset, int] = UNSET
    cpu: Union[Unset, float] = UNSET
    gateway: Union[Unset, Gateway] = UNSET
    git_commit: Union[Unset, str] = UNSET
    go: Union[Unset, str] = UNSET
    gomaxprocs: Union[Unset, int] = UNSET
    host: Union[Unset, str] = UNSET
    http_base_path: Union[Unset, str] = UNSET
    http_host: Union[Unset, str] = UNSET
    http_port: Union[Unset, int] = UNSET
    http_req_stats: Union[Unset, Any] = UNSET
    https_port: Union[Unset, int] = UNSET
    in_bytes: Union[Unset, int] = UNSET
    in_msgs: Union[Unset, int] = UNSET
    jetstream: Union[Unset, Jetstream] = UNSET
    leaf: Union[Unset, LeafNode] = UNSET
    leafnodes: Union[Unset, int] = UNSET
    max_connections: Union[Unset, int] = UNSET
    max_control_line: Union[Unset, int] = UNSET
    max_payload: Union[Unset, int] = UNSET
    max_pending: Union[Unset, int] = UNSET
    mem: Union[Unset, int] = UNSET
    now: Union[Unset, datetime.datetime] = UNSET
    out_bytes: Union[Unset, int] = UNSET
    out_msgs: Union[Unset, int] = UNSET
    ping_interval: Union[Unset, int] = UNSET
    ping_max: Union[Unset, int] = UNSET
    port: Union[Unset, int] = UNSET
    proto: Union[Unset, int] = UNSET
    remotes: Union[Unset, int] = UNSET
    routes: Union[Unset, int] = UNSET
    server_id: Union[Unset, str] = UNSET
    server_name: Union[Unset, str] = UNSET
    slow_consumers: Union[Unset, int] = UNSET
    start: Union[Unset, datetime.datetime] = UNSET
    subscriptions: Union[Unset, int] = UNSET
    system_account: Union[Unset, str] = UNSET
    tls_timeout: Union[Unset, int] = UNSET
    total_connections: Union[Unset, int] = UNSET
    uptime: Union[Unset, str] = UNSET
    version: Union[Unset, str] = UNSET
    write_deadline: Union[Unset, int] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        auth_timeout = self.auth_timeout
        if not isinstance(self.cluster, Unset):
            cluster = self.cluster
        config_load_time: Union[Unset, str] = UNSET
        if not isinstance(self.config_load_time, Unset):
            config_load_time = self.config_load_time.isoformat()
        connections = self.connections
        cores = self.cores
        cpu = self.cpu
        if not isinstance(self.gateway, Unset):
            gateway = self.gateway
        git_commit = self.git_commit
        go = self.go
        gomaxprocs = self.gomaxprocs
        host = self.host
        http_base_path = self.http_base_path
        http_host = self.http_host
        http_port = self.http_port
        http_req_stats = self.http_req_stats
        https_port = self.https_port
        in_bytes = self.in_bytes
        in_msgs = self.in_msgs
        if not isinstance(self.jetstream, Unset):
            jetstream = self.jetstream
        if not isinstance(self.leaf, Unset):
            leaf = self.leaf
        leafnodes = self.leafnodes
        max_connections = self.max_connections
        max_control_line = self.max_control_line
        max_payload = self.max_payload
        max_pending = self.max_pending
        mem = self.mem
        now: Union[Unset, str] = UNSET
        if not isinstance(self.now, Unset):
            now = self.now.isoformat()
        out_bytes = self.out_bytes
        out_msgs = self.out_msgs
        ping_interval = self.ping_interval
        ping_max = self.ping_max
        port = self.port
        proto = self.proto
        remotes = self.remotes
        routes = self.routes
        server_id = self.server_id
        server_name = self.server_name
        slow_consumers = self.slow_consumers
        start: Union[Unset, str] = UNSET
        if not isinstance(self.start, Unset):
            start = self.start.isoformat()
        subscriptions = self.subscriptions
        system_account = self.system_account
        tls_timeout = self.tls_timeout
        total_connections = self.total_connections
        uptime = self.uptime
        version = self.version
        write_deadline = self.write_deadline

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if auth_timeout is not UNSET:
            field_dict["auth_timeout"] = auth_timeout
        if cluster is not UNSET:
            field_dict["cluster"] = cluster
        if config_load_time is not UNSET:
            field_dict["config_load_time"] = config_load_time
        if connections is not UNSET:
            field_dict["connections"] = connections
        if cores is not UNSET:
            field_dict["cores"] = cores
        if cpu is not UNSET:
            field_dict["cpu"] = cpu
        if gateway is not UNSET:
            field_dict["gateway"] = gateway
        if git_commit is not UNSET:
            field_dict["git_commit"] = git_commit
        if go is not UNSET:
            field_dict["go"] = go
        if gomaxprocs is not UNSET:
            field_dict["gomaxprocs"] = gomaxprocs
        if host is not UNSET:
            field_dict["host"] = host
        if http_base_path is not UNSET:
            field_dict["http_base_path"] = http_base_path
        if http_host is not UNSET:
            field_dict["http_host"] = http_host
        if http_port is not UNSET:
            field_dict["http_port"] = http_port
        if http_req_stats is not UNSET:
            field_dict["http_req_stats"] = http_req_stats
        if https_port is not UNSET:
            field_dict["https_port"] = https_port
        if in_bytes is not UNSET:
            field_dict["in_bytes"] = in_bytes
        if in_msgs is not UNSET:
            field_dict["in_msgs"] = in_msgs
        if jetstream is not UNSET:
            field_dict["jetstream"] = jetstream
        if leaf is not UNSET:
            field_dict["leaf"] = leaf
        if leafnodes is not UNSET:
            field_dict["leafnodes"] = leafnodes
        if max_connections is not UNSET:
            field_dict["max_connections"] = max_connections
        if max_control_line is not UNSET:
            field_dict["max_control_line"] = max_control_line
        if max_payload is not UNSET:
            field_dict["max_payload"] = max_payload
        if max_pending is not UNSET:
            field_dict["max_pending"] = max_pending
        if mem is not UNSET:
            field_dict["mem"] = mem
        if now is not UNSET:
            field_dict["now"] = now
        if out_bytes is not UNSET:
            field_dict["out_bytes"] = out_bytes
        if out_msgs is not UNSET:
            field_dict["out_msgs"] = out_msgs
        if ping_interval is not UNSET:
            field_dict["ping_interval"] = ping_interval
        if ping_max is not UNSET:
            field_dict["ping_max"] = ping_max
        if port is not UNSET:
            field_dict["port"] = port
        if proto is not UNSET:
            field_dict["proto"] = proto
        if remotes is not UNSET:
            field_dict["remotes"] = remotes
        if routes is not UNSET:
            field_dict["routes"] = routes
        if server_id is not UNSET:
            field_dict["server_id"] = server_id
        if server_name is not UNSET:
            field_dict["server_name"] = server_name
        if slow_consumers is not UNSET:
            field_dict["slow_consumers"] = slow_consumers
        if start is not UNSET:
            field_dict["start"] = start
        if subscriptions is not UNSET:
            field_dict["subscriptions"] = subscriptions
        if system_account is not UNSET:
            field_dict["system_account"] = system_account
        if tls_timeout is not UNSET:
            field_dict["tls_timeout"] = tls_timeout
        if total_connections is not UNSET:
            field_dict["total_connections"] = total_connections
        if uptime is not UNSET:
            field_dict["uptime"] = uptime
        if version is not UNSET:
            field_dict["version"] = version
        if write_deadline is not UNSET:
            field_dict["write_deadline"] = write_deadline

        return field_dict

    @classmethod
    def from_dict(cls: Type[M], src_dict: Dict[str, Any]) -> M:
        d = src_dict.copy()
        auth_timeout = d.pop("auth_timeout", UNSET)

        _cluster = d.pop("cluster", UNSET)
        cluster: Union[Unset, Cluster]
        if isinstance(_cluster, Unset):
            cluster = UNSET
        else:
            cluster = Cluster(_cluster)

        _config_load_time = d.pop("config_load_time", UNSET)
        config_load_time: Union[Unset, datetime.datetime]
        if isinstance(_config_load_time, Unset):
            config_load_time = UNSET
        else:
            config_load_time = isoparse(_config_load_time)

        connections = d.pop("connections", UNSET)

        cores = d.pop("cores", UNSET)

        cpu = d.pop("cpu", UNSET)

        _gateway = d.pop("gateway", UNSET)
        gateway: Union[Unset, Gateway]
        if isinstance(_gateway, Unset):
            gateway = UNSET
        else:
            gateway = Gateway(_gateway)

        git_commit = d.pop("git_commit", UNSET)

        go = d.pop("go", UNSET)

        gomaxprocs = d.pop("gomaxprocs", UNSET)

        host = d.pop("host", UNSET)

        http_base_path = d.pop("http_base_path", UNSET)

        http_host = d.pop("http_host", UNSET)

        http_port = d.pop("http_port", UNSET)

        http_req_stats = d.pop("http_req_stats", UNSET)
        https_port = d.pop("https_port", UNSET)

        in_bytes = d.pop("in_bytes", UNSET)

        in_msgs = d.pop("in_msgs", UNSET)

        _jetstream = d.pop("jetstream", UNSET)
        jetstream: Union[Unset, Jetstream]
        if isinstance(_jetstream, Unset):
            jetstream = UNSET
        else:
            jetstream = Jetstream(_jetstream)

        _leaf = d.pop("leaf", UNSET)
        leaf: Union[Unset, LeafNode]
        if isinstance(_leaf, Unset):
            leaf = UNSET
        else:
            leaf = LeafNode(_leaf)

        leafnodes = d.pop("leafnodes", UNSET)

        max_connections = d.pop("max_connections", UNSET)

        max_control_line = d.pop("max_control_line", UNSET)

        max_payload = d.pop("max_payload", UNSET)

        max_pending = d.pop("max_pending", UNSET)

        mem = d.pop("mem", UNSET)

        _now = d.pop("now", UNSET)
        now: Union[Unset, datetime.datetime]
        if isinstance(_now, Unset):
            now = UNSET
        else:
            now = isoparse(_now)

        out_bytes = d.pop("out_bytes", UNSET)

        out_msgs = d.pop("out_msgs", UNSET)

        ping_interval = d.pop("ping_interval", UNSET)

        ping_max = d.pop("ping_max", UNSET)

        port = d.pop("port", UNSET)

        proto = d.pop("proto", UNSET)

        remotes = d.pop("remotes", UNSET)

        routes = d.pop("routes", UNSET)

        server_id = d.pop("server_id", UNSET)

        server_name = d.pop("server_name", UNSET)

        slow_consumers = d.pop("slow_consumers", UNSET)

        _start = d.pop("start", UNSET)
        start: Union[Unset, datetime.datetime]
        if isinstance(_start, Unset):
            start = UNSET
        else:
            start = isoparse(_start)

        subscriptions = d.pop("subscriptions", UNSET)

        system_account = d.pop("system_account", UNSET)

        tls_timeout = d.pop("tls_timeout", UNSET)

        total_connections = d.pop("total_connections", UNSET)

        uptime = d.pop("uptime", UNSET)

        version = d.pop("version", UNSET)

        write_deadline = d.pop("write_deadline", UNSET)

        connection = cls(
            auth_timeout=auth_timeout,
            cluster=cluster,
            config_load_time=config_load_time,
            connections=connections,
            cores=cores,
            cpu=cpu,
            gateway=gateway,
            git_commit=git_commit,
            go=go,
            gomaxprocs=gomaxprocs,
            host=host,
            http_base_path=http_base_path,
            http_host=http_host,
            http_port=http_port,
            http_req_stats=http_req_stats,
            https_port=https_port,
            in_bytes=in_bytes,
            in_msgs=in_msgs,
            jetstream=jetstream,
            leaf=leaf,
            leafnodes=leafnodes,
            max_connections=max_connections,
            max_control_line=max_control_line,
            max_payload=max_payload,
            max_pending=max_pending,
            mem=mem,
            now=now,
            out_bytes=out_bytes,
            out_msgs=out_msgs,
            ping_interval=ping_interval,
            ping_max=ping_max,
            port=port,
            proto=proto,
            remotes=remotes,
            routes=routes,
            server_id=server_id,
            server_name=server_name,
            slow_consumers=slow_consumers,
            start=start,
            subscriptions=subscriptions,
            system_account=system_account,
            tls_timeout=tls_timeout,
            total_connections=total_connections,
            uptime=uptime,
            version=version,
            write_deadline=write_deadline,
        )

        connection.additional_properties = d
        return connection

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.billing_info import BillingInfo
from ..models.card_details import CardDetails
from ..models.payment_method_type import PaymentMethodType
from ..types import UNSET, Unset

N = TypeVar("N", bound="PaymentMethod")


@attr.s(auto_attribs=True)
class PaymentMethod:
    """A payment method."""  # noqa: E501

    billing_info: Union[Unset, BillingInfo] = UNSET
    card: Union[Unset, CardDetails] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    id: Union[Unset, str] = UNSET
    metadata: Union[Unset, Any] = UNSET
    type: Union[Unset, PaymentMethodType] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        if not isinstance(self.billing_info, Unset):
            billing_info = self.billing_info
        if not isinstance(self.card, Unset):
            card = self.card
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()
        id = self.id
        metadata = self.metadata
        if not isinstance(self.type, Unset):
            type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if billing_info is not UNSET:
            field_dict["billing_info"] = billing_info
        if card is not UNSET:
            field_dict["card"] = card
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if id is not UNSET:
            field_dict["id"] = id
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[N], src_dict: Dict[str, Any]) -> N:
        d = src_dict.copy()
        _billing_info = d.pop("billing_info", UNSET)
        billing_info: Union[Unset, BillingInfo]
        if isinstance(_billing_info, Unset):
            billing_info = UNSET
        else:
            billing_info = BillingInfo(_billing_info)

        _card = d.pop("card", UNSET)
        card: Union[Unset, CardDetails]
        if isinstance(_card, Unset):
            card = UNSET
        else:
            card = CardDetails(_card)

        _created_at = d.pop("created_at", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        id = d.pop("id", UNSET)

        metadata = d.pop("metadata", UNSET)
        _type = d.pop("type", UNSET)
        type: Union[Unset, PaymentMethodType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = _type  # type: ignore[arg-type]

        payment_method = cls(
            billing_info=billing_info,
            card=card,
            created_at=created_at,
            id=id,
            metadata=metadata,
            type=type,
        )

        payment_method.additional_properties = d
        return payment_method

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

E = TypeVar("E", bound="Cluster")


@attr.s(auto_attribs=True)
class Cluster:
    """Cluster information."""  # noqa: E501

    addr: Union[Unset, str] = UNSET
    auth_timeout: Union[Unset, int] = UNSET
    cluster_port: Union[Unset, int] = UNSET
    name: Union[Unset, str] = UNSET
    tls_timeout: Union[Unset, int] = UNSET
    urls: Union[Unset, List[str]] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        addr = self.addr
        auth_timeout = self.auth_timeout
        cluster_port = self.cluster_port
        name = self.name
        tls_timeout = self.tls_timeout
        urls: Union[Unset, List[str]] = UNSET
        if not isinstance(self.urls, Unset):
            urls = self.urls

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if addr is not UNSET:
            field_dict["addr"] = addr
        if auth_timeout is not UNSET:
            field_dict["auth_timeout"] = auth_timeout
        if cluster_port is not UNSET:
            field_dict["cluster_port"] = cluster_port
        if name is not UNSET:
            field_dict["name"] = name
        if tls_timeout is not UNSET:
            field_dict["tls_timeout"] = tls_timeout
        if urls is not UNSET:
            field_dict["urls"] = urls

        return field_dict

    @classmethod
    def from_dict(cls: Type[E], src_dict: Dict[str, Any]) -> E:
        d = src_dict.copy()
        addr = d.pop("addr", UNSET)

        auth_timeout = d.pop("auth_timeout", UNSET)

        cluster_port = d.pop("cluster_port", UNSET)

        name = d.pop("name", UNSET)

        tls_timeout = d.pop("tls_timeout", UNSET)

        urls = cast(List[str], d.pop("urls", UNSET))

        cluster = cls(
            addr=addr,
            auth_timeout=auth_timeout,
            cluster_port=cluster_port,
            name=name,
            tls_timeout=tls_timeout,
            urls=urls,
        )

        cluster.additional_properties = d
        return cluster

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class AiPluginApiType(str, Enum):
    """AI plugin api type."""  # noqa: E501

    """# An OpenAPI specification. """  # noqa: E501
    OPENAPI = "openapi"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class UnitLength(str, Enum):
    """The valid types of length units."""  # noqa: E501

    """# Centimetres <https://en.wikipedia.org/wiki/Centimetre> """  # noqa: E501
    CENTIMETRES = "centimetres"
    """# Decimetres <https://en.wikipedia.org/wiki/Decimetre> """  # noqa: E501
    DECIMETRES = "decimetres"
    """# Feet <https://en.wikipedia.org/wiki/Foot_(unit)> """  # noqa: E501
    FEET = "feet"
    """# Furlongs <https://en.wikipedia.org/wiki/Furlong> """  # noqa: E501
    FURLONGS = "furlongs"
    """# Hectometres <https://en.wikipedia.org/wiki/Hectometre> """  # noqa: E501
    HECTOMETRES = "hectometres"
    """# Inches <https://en.wikipedia.org/wiki/Inch> """  # noqa: E501
    INCHES = "inches"
    """# Kilometres <https://en.wikipedia.org/wiki/Kilometre> """  # noqa: E501
    KILOMETRES = "kilometres"
    """# Metres <https://en.wikipedia.org/wiki/Metre> """  # noqa: E501
    METRES = "metres"
    """# Micrometres <https://en.wikipedia.org/wiki/Micrometre> """  # noqa: E501
    MICROMETRES = "micrometres"
    """# Miles <https://en.wikipedia.org/wiki/Mile> """  # noqa: E501
    MILES = "miles"
    """# Millimetres <https://en.wikipedia.org/wiki/Millimetre> """  # noqa: E501
    MILLIMETRES = "millimetres"
    """# Nanometres <https://en.wikipedia.org/wiki/Nanometre> """  # noqa: E501
    NANOMETRES = "nanometres"
    """# Yards <https://en.wikipedia.org/wiki/Yard> """  # noqa: E501
    YARDS = "yards"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict

import httpx

from ...client import Client
from ...types import Response


def _get_kwargs(
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/ws/modeling/commands".format(client.base_url)  # noqa: E501

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(*, response: httpx.Response):
    return


def _build_response(*, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
) -> Response[Any]:
    kwargs = _get_kwargs(
        client=client,
    )

    response = httpx.get(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
):
    """Pass those commands to the engine via websocket, and pass responses back to the client. Basically, this is a websocket proxy between the frontend/client and the engine."""  # noqa: E501

    return sync_detailed(
        client=client,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
) -> Response[Any]:
    kwargs = _get_kwargs(
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
):
    """Pass those commands to the engine via websocket, and pass responses back to the client. Basically, this is a websocket proxy between the frontend/client and the engine."""  # noqa: E501

    return (
        await asyncio_detailed(
            client=client,
        )
    ).parsed



from enum import Enum
from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.camera_drag_interaction_type import CameraDragInteractionType
from ..models.modeling_cmd_id import ModelingCmdId
from ..models.path_segment import PathSegment
from ..models.point2d import Point2d
from ..models.point3d import Point3d
from ..types import UNSET, Unset
from .extrude import Extrude


class StartPath(str, Enum):
    """Start a path."""  # noqa: E501

    START_PATH = "StartPath"

    def __str__(self) -> str:
        return str(self.value)


B = TypeVar("B", bound="MovePathPen")


@attr.s(auto_attribs=True)
class MovePathPen:
    path: Union[Unset, ModelingCmdId] = UNSET
    to: Union[Unset, Point3d] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        if not isinstance(self.path, Unset):
            path = self.path
        if not isinstance(self.to, Unset):
            to = self.to

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if path is not UNSET:
            field_dict["path"] = path
        if to is not UNSET:
            field_dict["to"] = to

        return field_dict

    @classmethod
    def from_dict(cls: Type[B], src_dict: Dict[str, Any]) -> B:
        d = src_dict.copy()
        _path = d.pop("path", UNSET)
        path: Union[Unset, ModelingCmdId]
        if isinstance(_path, Unset):
            path = UNSET
        else:
            path = ModelingCmdId(_path)

        _to = d.pop("to", UNSET)
        to: Union[Unset, Point3d]
        if isinstance(_to, Unset):
            to = UNSET
        else:
            to = Point3d(_to)

        move_path_pen = cls(
            path=path,
            to=to,
        )

        move_path_pen.additional_properties = d
        return move_path_pen

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties


B = TypeVar("B", bound="ExtendPath")


@attr.s(auto_attribs=True)
class ExtendPath:
    path: Union[Unset, ModelingCmdId] = UNSET
    segment: Union[Unset, PathSegment] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        if not isinstance(self.path, Unset):
            path = self.path
        if not isinstance(self.segment, Unset):
            segment = self.segment

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if path is not UNSET:
            field_dict["path"] = path
        if segment is not UNSET:
            field_dict["segment"] = segment

        return field_dict

    @classmethod
    def from_dict(cls: Type[B], src_dict: Dict[str, Any]) -> B:
        d = src_dict.copy()
        _path = d.pop("path", UNSET)
        path: Union[Unset, ModelingCmdId]
        if isinstance(_path, Unset):
            path = UNSET
        else:
            path = ModelingCmdId(_path)

        _segment = d.pop("segment", UNSET)
        segment: Union[Unset, PathSegment]
        if isinstance(_segment, Unset):
            segment = UNSET
        else:
            segment = _segment  # type: ignore[arg-type]

        extend_path = cls(
            path=path,
            segment=segment,
        )

        extend_path.additional_properties = d
        return extend_path

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties


P = TypeVar("P", bound="ClosePath")


@attr.s(auto_attribs=True)
class ClosePath:
    path_id: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        path_id = self.path_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if path_id is not UNSET:
            field_dict["path_id"] = path_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[P], src_dict: Dict[str, Any]) -> P:
        d = src_dict.copy()
        path_id = d.pop("path_id", UNSET)

        close_path = cls(
            path_id=path_id,
        )

        close_path.additional_properties = d
        return close_path

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties


J = TypeVar("J", bound="CameraDragStart")


@attr.s(auto_attribs=True)
class CameraDragStart:
    interaction: Union[Unset, CameraDragInteractionType] = UNSET
    window: Union[Unset, Point2d] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        if not isinstance(self.interaction, Unset):
            interaction = self.interaction
        if not isinstance(self.window, Unset):
            window = self.window

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if interaction is not UNSET:
            field_dict["interaction"] = interaction
        if window is not UNSET:
            field_dict["window"] = window

        return field_dict

    @classmethod
    def from_dict(cls: Type[J], src_dict: Dict[str, Any]) -> J:
        d = src_dict.copy()
        _interaction = d.pop("interaction", UNSET)
        interaction: Union[Unset, CameraDragInteractionType]
        if isinstance(_interaction, Unset):
            interaction = UNSET
        else:
            interaction = _interaction  # type: ignore[arg-type]

        _window = d.pop("window", UNSET)
        window: Union[Unset, Point2d]
        if isinstance(_window, Unset):
            window = UNSET
        else:
            window = Point2d(_window)

        camera_drag_start = cls(
            interaction=interaction,
            window=window,
        )

        camera_drag_start.additional_properties = d
        return camera_drag_start

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties


T = TypeVar("T", bound="CameraDragMove")


@attr.s(auto_attribs=True)
class CameraDragMove:
    interaction: Union[Unset, CameraDragInteractionType] = UNSET
    sequence: Union[Unset, int] = UNSET
    window: Union[Unset, Point2d] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        if not isinstance(self.interaction, Unset):
            interaction = self.interaction
        sequence = self.sequence
        if not isinstance(self.window, Unset):
            window = self.window

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if interaction is not UNSET:
            field_dict["interaction"] = interaction
        if sequence is not UNSET:
            field_dict["sequence"] = sequence
        if window is not UNSET:
            field_dict["window"] = window

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _interaction = d.pop("interaction", UNSET)
        interaction: Union[Unset, CameraDragInteractionType]
        if isinstance(_interaction, Unset):
            interaction = UNSET
        else:
            interaction = _interaction  # type: ignore[arg-type]

        sequence = d.pop("sequence", UNSET)

        _window = d.pop("window", UNSET)
        window: Union[Unset, Point2d]
        if isinstance(_window, Unset):
            window = UNSET
        else:
            window = Point2d(_window)

        camera_drag_move = cls(
            interaction=interaction,
            sequence=sequence,
            window=window,
        )

        camera_drag_move.additional_properties = d
        return camera_drag_move

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties


V = TypeVar("V", bound="CameraDragEnd")


@attr.s(auto_attribs=True)
class CameraDragEnd:
    interaction: Union[Unset, CameraDragInteractionType] = UNSET
    window: Union[Unset, Point2d] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        if not isinstance(self.interaction, Unset):
            interaction = self.interaction
        if not isinstance(self.window, Unset):
            window = self.window

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if interaction is not UNSET:
            field_dict["interaction"] = interaction
        if window is not UNSET:
            field_dict["window"] = window

        return field_dict

    @classmethod
    def from_dict(cls: Type[V], src_dict: Dict[str, Any]) -> V:
        d = src_dict.copy()
        _interaction = d.pop("interaction", UNSET)
        interaction: Union[Unset, CameraDragInteractionType]
        if isinstance(_interaction, Unset):
            interaction = UNSET
        else:
            interaction = _interaction  # type: ignore[arg-type]

        _window = d.pop("window", UNSET)
        window: Union[Unset, Point2d]
        if isinstance(_window, Unset):
            window = UNSET
        else:
            window = Point2d(_window)

        camera_drag_end = cls(
            interaction=interaction,
            window=window,
        )

        camera_drag_end.additional_properties = d
        return camera_drag_end

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties


ModelingCmd = Union[
    StartPath,
    MovePathPen,
    ExtendPath,
    Extrude,
    ClosePath,
    CameraDragStart,
    CameraDragMove,
    CameraDragEnd,
]



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.ai_plugin_api_type import AiPluginApiType
from ..types import UNSET, Unset

S = TypeVar("S", bound="AiPluginApi")


@attr.s(auto_attribs=True)
class AiPluginApi:
    """AI plugin api information."""  # noqa: E501

    is_user_authenticated: Union[Unset, bool] = False
    type: Union[Unset, AiPluginApiType] = UNSET
    url: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        is_user_authenticated = self.is_user_authenticated
        if not isinstance(self.type, Unset):
            type = self.type
        url = self.url

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if is_user_authenticated is not UNSET:
            field_dict["is_user_authenticated"] = is_user_authenticated
        if type is not UNSET:
            field_dict["type"] = type
        if url is not UNSET:
            field_dict["url"] = url

        return field_dict

    @classmethod
    def from_dict(cls: Type[S], src_dict: Dict[str, Any]) -> S:
        d = src_dict.copy()
        is_user_authenticated = d.pop("is_user_authenticated", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, AiPluginApiType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = _type  # type: ignore[arg-type]

        url = d.pop("url", UNSET)

        ai_plugin_api = cls(
            is_user_authenticated=is_user_authenticated,
            type=type,
            url=url,
        )

        ai_plugin_api.additional_properties = d
        return ai_plugin_api

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

H = TypeVar("H", bound="Mesh")


@attr.s(auto_attribs=True)
class Mesh:
    mesh: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        mesh = self.mesh

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if mesh is not UNSET:
            field_dict["mesh"] = mesh

        return field_dict

    @classmethod
    def from_dict(cls: Type[H], src_dict: Dict[str, Any]) -> H:
        d = src_dict.copy()
        mesh = d.pop("mesh", UNSET)

        mesh = cls(
            mesh=mesh,
        )

        mesh.additional_properties = d
        return mesh

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class AiPluginHttpAuthType(str, Enum):
    """AI plugin http auth type."""  # noqa: E501

    """# Basic. """  # noqa: E501
    BASIC = "basic"
    """# Bearer. """  # noqa: E501
    BEARER = "bearer"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.api_call_status import ApiCallStatus
from ...models.async_api_call_results_page import AsyncApiCallResultsPage
from ...models.created_at_sort_mode import CreatedAtSortMode
from ...models.error import Error
from ...types import Response


def _get_kwargs(
    sort_by: CreatedAtSortMode,
    status: ApiCallStatus,
    *,
    client: Client,
    limit: Optional[int] = None,
    page_token: Optional[str] = None,
) -> Dict[str, Any]:
    url = "{}/async/operations".format(client.base_url)  # noqa: E501
    if limit is not None:
        if "?" in url:
            url = url + "&limit=" + str(limit)
        else:
            url = url + "?limit=" + str(limit)
    if page_token is not None:
        if "?" in url:
            url = url + "&page_token=" + str(page_token)
        else:
            url = url + "?page_token=" + str(page_token)
    if sort_by is not None:
        if "?" in url:
            url = url + "&sort_by=" + str(sort_by)
        else:
            url = url + "?sort_by=" + str(sort_by)
    if status is not None:
        if "?" in url:
            url = url + "&status=" + str(status)
        else:
            url = url + "?status=" + str(status)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[AsyncApiCallResultsPage, Error]]:
    if response.status_code == 200:
        response_200 = AsyncApiCallResultsPage.from_dict(response.json())
        return response_200
    if response.status_code == 400:
        response_4XX = Error.from_dict(response.json())
        return response_4XX
    if response.status_code == 500:
        response_5XX = Error.from_dict(response.json())
        return response_5XX
    return Error.from_dict(response.json())


def _build_response(
    *, response: httpx.Response
) -> Response[Optional[Union[AsyncApiCallResultsPage, Error]]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    sort_by: CreatedAtSortMode,
    status: ApiCallStatus,
    *,
    client: Client,
    limit: Optional[int] = None,
    page_token: Optional[str] = None,
) -> Response[Optional[Union[AsyncApiCallResultsPage, Error]]]:
    kwargs = _get_kwargs(
        limit=limit,
        page_token=page_token,
        sort_by=sort_by,
        status=status,
        client=client,
    )

    response = httpx.get(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    sort_by: CreatedAtSortMode,
    status: ApiCallStatus,
    *,
    client: Client,
    limit: Optional[int] = None,
    page_token: Optional[str] = None,
) -> Optional[Union[AsyncApiCallResultsPage, Error]]:
    """For async file conversion operations, this endpoint does not return the contents of converted files (`output`). To get the contents use the `/async/operations/{id}` endpoint.
    This endpoint requires authentication by a KittyCAD employee."""  # noqa: E501

    return sync_detailed(
        limit=limit,
        page_token=page_token,
        sort_by=sort_by,
        status=status,
        client=client,
    ).parsed


async def asyncio_detailed(
    sort_by: CreatedAtSortMode,
    status: ApiCallStatus,
    *,
    client: Client,
    limit: Optional[int] = None,
    page_token: Optional[str] = None,
) -> Response[Optional[Union[AsyncApiCallResultsPage, Error]]]:
    kwargs = _get_kwargs(
        limit=limit,
        page_token=page_token,
        sort_by=sort_by,
        status=status,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    sort_by: CreatedAtSortMode,
    status: ApiCallStatus,
    *,
    client: Client,
    limit: Optional[int] = None,
    page_token: Optional[str] = None,
) -> Optional[Union[AsyncApiCallResultsPage, Error]]:
    """For async file conversion operations, this endpoint does not return the contents of converted files (`output`). To get the contents use the `/async/operations/{id}` endpoint.
    This endpoint requires authentication by a KittyCAD employee."""  # noqa: E501

    return (
        await asyncio_detailed(
            limit=limit,
            page_token=page_token,
            sort_by=sort_by,
            status=status,
            client=client,
        )
    ).parsed



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.error import Error
from ...models.file_export_format import FileExportFormat
from ...models.mesh import Mesh
from ...types import Response


def _get_kwargs(
    output_format: FileExportFormat,
    prompt: str,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/ai/text-to-3d/{output_format}".format(
        client.base_url, output_format=output_format
    )  # noqa: E501
    if prompt is not None:
        if "?" in url:
            url = url + "&prompt=" + str(prompt)
        else:
            url = url + "?prompt=" + str(prompt)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[Mesh, Error]]:
    if response.status_code == 200:
        response_200 = Mesh.from_dict(response.json())
        return response_200
    if response.status_code == 400:
        response_4XX = Error.from_dict(response.json())
        return response_4XX
    if response.status_code == 500:
        response_5XX = Error.from_dict(response.json())
        return response_5XX
    return Error.from_dict(response.json())


def _build_response(
    *, response: httpx.Response
) -> Response[Optional[Union[Mesh, Error]]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    output_format: FileExportFormat,
    prompt: str,
    *,
    client: Client,
) -> Response[Optional[Union[Mesh, Error]]]:
    kwargs = _get_kwargs(
        output_format=output_format,
        prompt=prompt,
        client=client,
    )

    response = httpx.post(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    output_format: FileExportFormat,
    prompt: str,
    *,
    client: Client,
) -> Optional[Union[Mesh, Error]]:
    """This is an alpha endpoint. It will change in the future. The current output is honestly pretty bad. So if you find this endpoint, you get what you pay for, which currently is nothing. But in the future will be made a lot better."""  # noqa: E501

    return sync_detailed(
        output_format=output_format,
        prompt=prompt,
        client=client,
    ).parsed


async def asyncio_detailed(
    output_format: FileExportFormat,
    prompt: str,
    *,
    client: Client,
) -> Response[Optional[Union[Mesh, Error]]]:
    kwargs = _get_kwargs(
        output_format=output_format,
        prompt=prompt,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    output_format: FileExportFormat,
    prompt: str,
    *,
    client: Client,
) -> Optional[Union[Mesh, Error]]:
    """This is an alpha endpoint. It will change in the future. The current output is honestly pretty bad. So if you find this endpoint, you get what you pay for, which currently is nothing. But in the future will be made a lot better."""  # noqa: E501

    return (
        await asyncio_detailed(
            output_format=output_format,
            prompt=prompt,
            client=client,
        )
    ).parsed



from enum import Enum


class SystemInfoCgroupDriverEnum(str, Enum):
    EMPTY = ""
    CGROUPFS = "cgroupfs"
    SYSTEMD = "systemd"
    NONE = "none"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.error import Error
from ...models.file_conversion import FileConversion
from ...models.file_export_format import FileExportFormat
from ...models.file_import_format import FileImportFormat
from ...types import Response


def _get_kwargs(
    output_format: FileExportFormat,
    src_format: FileImportFormat,
    body: bytes,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/file/conversion/{src_format}/{output_format}".format(
        client.base_url, output_format=output_format, src_format=src_format
    )  # noqa: E501

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "content": body,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[FileConversion, Error]]:
    if response.status_code == 201:
        response_201 = FileConversion.from_dict(response.json())
        return response_201
    if response.status_code == 400:
        response_4XX = Error.from_dict(response.json())
        return response_4XX
    if response.status_code == 500:
        response_5XX = Error.from_dict(response.json())
        return response_5XX
    return Error.from_dict(response.json())


def _build_response(
    *, response: httpx.Response
) -> Response[Optional[Union[FileConversion, Error]]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    output_format: FileExportFormat,
    src_format: FileImportFormat,
    body: bytes,
    *,
    client: Client,
) -> Response[Optional[Union[FileConversion, Error]]]:
    kwargs = _get_kwargs(
        output_format=output_format,
        src_format=src_format,
        body=body,
        client=client,
    )

    response = httpx.post(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    output_format: FileExportFormat,
    src_format: FileImportFormat,
    body: bytes,
    *,
    client: Client,
) -> Optional[Union[FileConversion, Error]]:
    """If you wish to specify the conversion options, use the `/file/conversion` endpoint instead.
    Convert a CAD file from one format to another. If the file being converted is larger than 25MB, it will be performed asynchronously.
    If the conversion is performed synchronously, the contents of the converted file (`output`) will be returned as a base64 encoded string.
    If the operation is performed asynchronously, the `id` of the operation will be returned. You can use the `id` returned from the request to get status information about the async operation from the `/async/operations/{id}` endpoint."""  # noqa: E501

    return sync_detailed(
        output_format=output_format,
        src_format=src_format,
        body=body,
        client=client,
    ).parsed


async def asyncio_detailed(
    output_format: FileExportFormat,
    src_format: FileImportFormat,
    body: bytes,
    *,
    client: Client,
) -> Response[Optional[Union[FileConversion, Error]]]:
    kwargs = _get_kwargs(
        output_format=output_format,
        src_format=src_format,
        body=body,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    output_format: FileExportFormat,
    src_format: FileImportFormat,
    body: bytes,
    *,
    client: Client,
) -> Optional[Union[FileConversion, Error]]:
    """If you wish to specify the conversion options, use the `/file/conversion` endpoint instead.
    Convert a CAD file from one format to another. If the file being converted is larger than 25MB, it will be performed asynchronously.
    If the conversion is performed synchronously, the contents of the converted file (`output`) will be returned as a base64 encoded string.
    If the operation is performed asynchronously, the `id` of the operation will be returned. You can use the `id` returned from the request to get status information about the async operation from the `/async/operations/{id}` endpoint."""  # noqa: E501

    return (
        await asyncio_detailed(
            output_format=output_format,
            src_format=src_format,
            body=body,
            client=client,
        )
    ).parsed
